import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;

import org.jdesktop.animation.timing.Animator;
import org.jdesktop.animation.timing.TimingTarget;
import org.jdesktop.animation.timing.TimingTargetAdapter;

public class FlowAnimatedLayout extends FlowLayout {
    private Container target;
    private boolean flag = false;

    public FlowAnimatedLayout() {
        super();

        ActionListener listener = new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                if (target != null) {
                    layoutContainer2(target);
                    target = null;
                }
            }
        };
        new Timer(1000, listener).start();
    }

    public FlowAnimatedLayout(int align) {
        super(align);

        ActionListener listener = new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                if (target != null) {
                    layoutContainer2(target);
                    target = null;
                }
            }
        };
        new Timer(1000, listener).start();
    }

    public FlowAnimatedLayout(int align, int hgap, int vgap) {
        super(align, hgap, vgap);

        ActionListener listener = new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                if (target != null) {
                    layoutContainer2(target);
                    target = null;
                }
            }
        };
        new Timer(1000, listener).start();
    }

    private int moveComponents(Container target, 
                               int x, int y, int width, int height,
                               int rowStart, int rowEnd, boolean ltr,
                               boolean useBaseline, int[] ascent,
                               int[] descent) {
        switch (getAlignment()) {
        case LEFT:
            x += ltr ? 0 : width;
            break;
        case CENTER:
            x += width / 2;
            break;
        case RIGHT:
            x += ltr ? width : 0;
            break;
        case LEADING:
            break;
        case TRAILING:
            x += width;
            break;
        }
        
        int maxAscent = 0;
        int nonbaselineHeight = 0;
        int baselineOffset = 0;
        if (useBaseline) {
            int maxDescent = 0;
            for (int i = rowStart ; i < rowEnd ; i++) {
                Component m = target.getComponent(i);
                if (m.isVisible()) {
                    if (ascent[i] >= 0) {
                        maxAscent = Math.max(maxAscent, ascent[i]);
                        maxDescent = Math.max(maxDescent, descent[i]);
                    }
                    else {
                        nonbaselineHeight = Math.max(m.getHeight(),
                                                     nonbaselineHeight);
                    }
                }
            }
            height = Math.max(maxAscent + maxDescent, nonbaselineHeight);
            baselineOffset = (height - maxAscent - maxDescent) / 2;
        }
	for (int i = rowStart ; i < rowEnd ; i++) {
            Component m = target.getComponent(i);
            if (m.isVisible()) {
                int cy;
                if (useBaseline && ascent[i] >= 0) {
                    cy = y + baselineOffset + maxAscent - ascent[i];
                }
                else {
                    cy = y + (height - m.getHeight()) / 2;
                }
                if (ltr) {
                    //                    m.setLocation(x, cy);
                    moveComponent(m, x, cy);
                } else {
                    //m.setLocation(target.getWidth() - x - m.getWidth(), cy);
                    moveComponent(m, target.getWidth() - x - m.getWidth(), cy);
                }
                x += m.getWidth() + getHgap();
	    }
	}
        return height;
    }

    public void moveComponent(final Component m, final int x, final int y) {
        final int preX = m.getX();
        final int preY = m.getY();

        TimingTarget target = new TimingTargetAdapter() {
                public void timingEvent(float fraction) {
                    int newX = (int)((x - preX) * fraction + preX);
                    int newY = (int)((y - preY) * fraction + preY);
                    m.setLocation(newX, newY);
                }
            };
        Animator animator = new Animator(400, target);
        animator.start();
    }

    public void layoutContainer(Container target) {
        if (flag == false) {
            flag = true;
            layoutContainer2(target);
        } else {
            this.target = target;
        }
    }

    public void layoutContainer2(Container target) {
        synchronized (target.getTreeLock()) {
            Insets insets = target.getInsets();
            int maxwidth = target.getWidth() - (insets.left + insets.right + getHgap()*2);
            int nmembers = target.getComponentCount();
            int x = 0, y = insets.top + getVgap();
            int rowh = 0, start = 0;
            
            boolean ltr = target.getComponentOrientation().isLeftToRight();
            
            boolean useBaseline = getAlignOnBaseline();
            int[] ascent = null;
            int[] descent = null;
            
            if (useBaseline) {
                ascent = new int[nmembers];
                descent = new int[nmembers];
            }
            
            for (int i = 0 ; i < nmembers ; i++) {
                Component m = target.getComponent(i);
                if (m.isVisible()) {
                    Dimension d = m.getPreferredSize();
                    m.setSize(d.width, d.height);
                    
                    if (useBaseline) {
                        int baseline = m.getBaseline(d.width, d.height);
                        if (baseline >= 0) {
                            ascent[i] = baseline;
                            descent[i] = d.height - baseline;
                        }
                        else {
                            ascent[i] = -1;
                        }
                    }
                    if ((x == 0) || ((x + d.width) <= maxwidth)) {
                        if (x > 0) {
                            x += getHgap();
                        }
                        x += d.width;
                        rowh = Math.max(rowh, d.height);
                    } else {
                        rowh = moveComponents(target, insets.left + getHgap(), y,
                                              maxwidth - x, rowh, start, i, ltr,
                                              useBaseline, ascent, descent);
                        x = d.width;
                        y += getVgap() + rowh;
                        rowh = d.height;
                        start = i;
                    }
                }
            }
            moveComponents(target, insets.left + getHgap(), y, maxwidth - x, rowh,
                           start, nmembers, ltr, useBaseline, ascent, descent);
        }
    }
}