import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Formatter;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.synth.SynthLookAndFeel;

public class ImageViewer {
    private java.util.List<ImagePanel> images;
    private boolean isReflected = false;
    private JFrame frame;

    public ImageViewer(String filename) {
        frame = new JFrame("Test");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        if (filename != null) {
            loadLaF(filename);
        }

        initMenu(frame);
        initSearchPanel(frame);
        images = initThumbPanel(frame);

        frame.setSize(480, 600);
        frame.setVisible(true);
    }

    private void loadLaF(String filename) {
        try {
            SynthLookAndFeel laf = new SynthLookAndFeel();
            laf.load(ImageViewer.class.getResourceAsStream(filename), 
                     ImageViewer.class);
            UIManager.setLookAndFeel(laf);
            SwingUtilities.updateComponentTreeUI(frame);
        } catch(ParseException ex) {
            ex.printStackTrace();
        } catch(UnsupportedLookAndFeelException ex) {
            ex.printStackTrace();
        } catch(IllegalArgumentException ex) {
            //            ex.printStackTrace();
        }
    }

    private void initMenu(JFrame frame) {
        JMenuBar bar = new JMenuBar();
        JMenu menu = new JMenu("Option");

        JMenuItem item = new JMenuItem("Reflection");
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    if (isReflected) {
                        for (ImagePanel image: images) {
                            image.setReflected(false);
                            image.repaint();
                        }
                        ((JMenuItem)event.getSource()).setText("Reflection");
                        isReflected = false;
                    } else {
                        for (ImagePanel image: images) {
                            image.setReflected(true);
                            image.repaint();
                        }
                        ((JMenuItem)event.getSource()).setText("No Reflection");
                        isReflected = true;
                    }
                }
            });
        menu.add(item);

        bar.add(menu);
        frame.setJMenuBar(bar);
    }

    private void initSearchPanel(JFrame frame) {
        JPanel panel = new JPanel();
        
        JLabel label = new JLabel("Keyword");
        panel.add(label);
        JTextField field = new JTextField(20);
        panel.add(field);
        JButton button = new JButton("Search");
        button.setFocusable(false);
        panel.add(button);

        GroupLayout layout = new GroupLayout(panel);
        panel.setLayout(layout);

        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        GroupLayout.SequentialGroup hGroup
            = layout.createSequentialGroup();
        hGroup.addComponent(label)
              .addComponent(field)
              .addComponent(button);

        layout.setHorizontalGroup(hGroup);
 
        GroupLayout.ParallelGroup vGroup
            = layout.createParallelGroup(GroupLayout.Alignment.BASELINE);
        vGroup.addComponent(label)
            .addComponent(field)
            .addComponent(button);
  
        layout.setVerticalGroup(vGroup);

        frame.add(panel, BorderLayout.NORTH);
    }

    private java.util.List<ImagePanel> initThumbPanel(JFrame frame) {
        java.util.List<ImagePanel> imagePanels 
            = new ArrayList<ImagePanel>();

        JPanel panel = new JPanel();
        panel.setName("thumb");

        FlowAnimatedLayout layout 
            = new FlowAnimatedLayout(FlowAnimatedLayout.LEFT);
        panel.setLayout(layout);
        
        for (int i = 0; i < 10; i++) {
            Formatter formatter = new Formatter();
            String filename 
                = formatter.format("images/thumb%02d.jpg", i).toString();
            try {
                ImagePanel imagePanel = new ImagePanel(filename);
                panel.add(imagePanel);
                imagePanels.add(imagePanel);
            } catch (IOException ex) {}
        }

        frame.add(panel, BorderLayout.CENTER);

        return imagePanels;
    }

    public static void main(final String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    if (args.length == 0) {
                        new ImageViewer(null);
                    } else {
                        new ImageViewer(args[0]);
                    }
                }
            });
    }
}